<?php

namespace App\Http\Controllers\Api;

use App\Http\Resources\Order as OrderResource;
use GlobalPayments\Api\Entities\Exceptions\ApiException;
use GlobalPayments\Api\Utils\GenerationUtils;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Resources\UserCollection;
use App\Models\Order;
use App\Models\Pub as Pub;
use GlobalPayments\Api\ServiceConfigs\Gateways\GpEcomConfig;
use GlobalPayments\Api\HostedPaymentConfig;
use GlobalPayments\Api\Entities\Enums\HppVersion;
use GlobalPayments\Api\Services\HostedService;
use GlobalPayments\Api\ServicesConfig;
use GlobalPayments\Api\Entities\Address;
use GlobalPayments\Api\Entities\Enums\AddressType;
use GlobalPayments\Api\Entities\HostedPaymentData;
use Illuminate\Support\Facades\Auth;
use GlobalPayments\Api\Entities\Enums\ChallengeRequestIndicator;
use Hash;

class PaymentController extends Controller {

    private function getService() {
        // configure client, request and HPP settings
        $config = new ServicesConfig();
        $config->merchantId = getCustomEnvironmentVariable('GLOBALPAYMENTS_MERCHANT_ID');
        $config->accountId = getCustomEnvironmentVariable('GLOBALPAYMENTS_ACCOUNT_ID');
        $config->sharedSecret = getCustomEnvironmentVariable('GLOBALPAYMENTS_SECRET');
        $config->serviceUrl = getCustomEnvironmentVariable('GLOBALPAYMENTS_ENDPOINT');

        $config->hostedPaymentConfig = new HostedPaymentConfig();
        $config->hostedPaymentConfig->version = HppVersion::VERSION_2;
        $config->hostedPaymentConfig->displaySavedCards = true;
        return new HostedService($config);
    }

    public function generateFields(Order $order) {
        $amount = $order->getTotalPayable();

        if (!$amount) {
            throw new \Exception("No payment required");
        }

        $service = $this->getService();

        $pub = Pub::where('id', $order->pub_id)->pluck('HIL_payer_no')->first();

        $hostedPaymentData = new HostedPaymentData();
        // Save cards
        $hostedPaymentData->offerToSaveCard = true;
        $hostedPaymentData->customerExists = true;

        $customerId = Auth::user()->id;
        $customerKey = $pub . '-' . str_replace(' ', '-', $customerId);
        $customerKey = $this->clean($customerKey);
        $hostedPaymentData->customerKey = $customerKey;

        // Force 3d secure
        $hostedPaymentData->challengeRequest = ChallengeRequestIndicator::CHALLENGE_MANDATED;

        $data = $service
            ->charge($amount)
            ->withCurrency("EUR")
            ->withOrderId($order->id)
            ->withCustomData('Hello World')
            ->withCustomerId($pub)
            ->withHostedPaymentData($hostedPaymentData)
            ->serialize();

        $data = json_decode($data);
        $data->endpoint = getCustomEnvironmentVariable('GLOBALPAYMENTS_ENDPOINT');

        return $data;
    }

    public function clean($string) {
        $string = str_replace(' ', '-', $string); // Replaces all spaces with hyphens.
     
        return preg_replace('/[^A-Za-z0-9\-]/', '', $string); // Removes special chars.
    }


    public function responseHandler(Order $order, Request $request) {
        $responseJson = json_encode($_POST);

        try {
            // create the response object from the response JSON
            $parsedResponse = $this->getService()->parseResponse($responseJson, isset($_POST['encoded']));

            // All good
            $reference = $parsedResponse->transactionReference->transactionId;

            if ($parsedResponse->responseCode === "00") {

                // Update order
                $order->update([
                    'reca_status' => 'complete'
                ]);

                $order->reca()->updateOrCreate([], [
                    'globalpay_id' => $reference,
                    'total' => $parsedResponse->authorizedAmount
                ]);

                // Get order
                foreach($order->suborders AS $suborder) {
                    (new OrderController())->markSuborderAsApproved($suborder);
                }

                return new OrderResource($order);
            } else {
                // Update order
                $order->update([
                    'reca_status' => 'failed'
                ]);

                $order->reca()->updateOrCreate([], [
                    'globalpay_status' => $parsedResponse->responseMessage,
                    'globalpay_id' => $reference
                ]);

                throw new \Exception("Payment failed");
            }
        } catch (ApiException $e) {
            throw $e;
        } catch (\Exception $e) {
            throw $e;
        }
    }

}